@@ -9,7 +9,6 @@ from ..helper import is_iterable
 from ..exceptions import RuleError
 from ..helper import NO_THING
 from enum import IntEnum
-from abc import ABC
 # import wrapt
 
 
@@ -29,10 +28,11 @@ class DecFuncEnum(IntEnum):
     def __str__(self):
         return self._name_
 
-class _DecBase(ABC):
+class _DecBase(object):
     _rx_star = re.compile("^\*(\d*)$")
 
     def __init__(self, **kwargs):
+        self._signature = None
         self._ftype: DecFuncEnum = kwargs.get("ftype", None)
         if self._ftype is not None:
             if not isinstance(self._ftype, DecFuncEnum):
@@ -44,8 +44,11 @@ class _DecBase(ABC):
         else:
             self._ftype = DecFuncEnum.FUNCTION
 
-    def _is_placeholder_arg(self, arg_name: str):
-        return _DecBase._rx_star.match(arg_name)
+    def _is_placeholder_arg(self, arg_name: str) -> bool:
+        m = _DecBase._rx_star.match(arg_name)
+        if m:
+            return True
+        return False
 
     def _drop_arg_first(self) -> bool:
         return self._ftype.value > DecFuncEnum.METHOD_STATIC.value
@@ -55,34 +58,28 @@ class _DecBase(ABC):
             return args[1:]
         return args
 
-    def _get_arg_names(self, func):
-        # arg name will only contain argument names and not *args or **kwargs
-        # if func starts with *args eg: foo(*args, one, two) then
-        # return will be empty list
-        argnames = func.__code__.co_varnames[:func.__code__.co_argcount]
-        return argnames
-
-    def _get_args_dict(self, method, args, kwargs):
+    def _get_signature(self, func) -> Signature:
+        if self._signature is None:
+            self._signature = signature(func)
+        return self._signature
+    
+    def _get_args_dict(self, func, args, kwargs) -> Dict[str, object]:
         # Positional argument cannot appear after keyword arguments
         # no *args after **kwargs def foo(**kwargs, *args) not valid
         # def foo(name, age, *args) not valid
         #
         # When only args are passed in (no **kwargs present) with named args then
         # the last named args with defaults are like additional args
-        sig = signature(method)
+        sig = self._get_signature(func)
         name_values = []
         i = 0
-        args_first = False
+        args_pos = -1
         drop_first = self._drop_arg_first()
-        if drop_first and len(args) > 0:
-            _args = args[1:]
-        else:
-            _args = args
-        arg_len = len(_args)
         for k, v in sig.parameters.items():
-            if v.kind == _ParameterKind.VAR_POSITIONAL:  # args, can only be in first postition
-                if (drop_first is True and i == 1) or i == 0:
-                    args_first = True
+            if v.kind == _ParameterKind.VAR_POSITIONAL:  # args
+                args_pos = i
+                if drop_first:
+                    args_pos -= 1
                 continue
             if v.kind == _ParameterKind.VAR_KEYWORD:  # kwargs
                 continue
@@ -91,31 +88,73 @@ class _DecBase(ABC):
             else:
                 name_values.append((k, NO_THING))
             i += 1
-        if drop_first and i > 0:
+        arg_offset = 0 if args_pos == -1 else args_pos
+        if drop_first and len(args) > arg_offset:
+            _args = args[(arg_offset + 1):]
+        else:
+            _args = args[arg_offset:]
+        arg_len = len(_args)
+        name_values_len = len(name_values)
+        if drop_first and name_values_len > 0:
             del name_values[0]
+            name_values_len -= 1
         offset = 0
-        if args_first is True and len(name_values) > 0:
+        if args_pos >= 0 and name_values_len > 0:
+            # count how many name, values from default
+            # are default values
             reversed_list = list(reversed(name_values))
             for k, v in reversed_list:
                 if not v is NO_THING:
                     offset += 1
                 else:
                     break
+
         name_defaults = {}
-        if args_first is True:
+        # add first keys of name_values
+        if args_pos > 0:
+            # there are name values before *args in function
+            # add first args so they are in the same order as entered.
+            for j in range(args_pos):
+                nv = name_values[j]
+                name_defaults[nv[0]] = nv[1]
+        # add args
+        if args_pos >= 0:
+            # add args that are positional arguments
             for j, arg in enumerate(_args):
-                key = "*" + str(j)
+                key = "*" + str(j + arg_offset)
                 name_defaults[key] = arg
-        for k, v in name_values:
-            name_defaults[k] = v
-        argnames = []
-        for j in range(len(name_values) - offset):
-            el = name_values[j]
-            argnames.append(el[0])
-        if args_first is False and arg_len > 0:
-            d = {**dict(zip(argnames, _args[:len(argnames)]))}
+
+        if args_pos > 0:
+            # add any remaining keys for name_values
+            remaining = len(name_values) - args_pos
+            if remaining > 0:
+                for j in range(remaining):
+                    index = j + args_pos
+                    nv = name_values[index]
+                    name_defaults[nv[0]] = nv[1]
+        else:
+            # there were no positional args before *args keyword so
+            # all all the name, values after postitioan args.
+            for k, v in name_values:
+                name_defaults[k] = v
+        if args_pos != 0 and arg_len > 0:
+            # if *args are not in position 0 and there are *args then
+            # make a dictionary of name values and update the
+            # name_defaults dictionary.
+            argnames = []
+            for j in range(len(name_values) - offset):
+                el = name_values[j]
+                argnames.append(el[0])
+            if drop_first:
+                _zip_args = args[1:]
+            else:
+                _zip_args = args
+            d = {**dict(zip(argnames, _zip_args[:len(argnames)]))}
             name_defaults.update(d)
         if len(kwargs) > 0:
+            # update name_default with any kwargs that are passed in.
+            # this will update any default values with the values passed
+            # into function at call time.
             name_defaults.update(kwargs)
         return name_defaults
 
@@ -143,6 +182,29 @@ class _DecBase(ABC):
             ord = {1: 'st', 2: 'nd', 3: 'rd'}.get(num % 10, 'th')
             return '{0}{1}'.format(num, ord)
 
+    def _get_star_args_pos(self, func) -> int:
+        """
+        Gets the zero base postion of *args in a function.
+
+        Args:
+            func (callable): function to get *args postion of.
+
+        Returns:
+            int: -1 if  *args not present; Otherwise zero based postion of *args
+        """
+        sig = self._get_signature(func)
+        args_pos = -1
+        drop_first = self._drop_arg_first()
+        i = 0
+        for k, v in sig.parameters.items():
+            if v.kind == _ParameterKind.VAR_POSITIONAL:  # args
+                args_pos = i
+                if drop_first:
+                    args_pos -= 1
+                break
+            i += 1
+        return args_pos
+
 class _RuleBase(_DecBase):
     def _get_err(self, fn: callable, e: RuleError):
         err = RuleError.from_rule_error(e, fn_name=fn.__name__)
@@ -242,6 +304,15 @@ class AcceptedTypes(_DecBase):
             self._kwargs = {**kwargs}
         else:
             self._kwargs = {}
+
+    def _get_formated_types(self, types: Iterator[type]) -> str:
+        result = ''
+        for i, t in enumerate(types):
+            if i > 0:
+                result = result + ' | '
+            result = f"{result}{t}"
+        return result
+
     
     def __call__(self, func: callable):
         @functools.wraps(func)
@@ -805,7 +876,7 @@ class DefaultArgs(object):
         Keyword Arguments:
             kwargs (Dict[str, object]): One or more Key, Value pairs to assign to wrapped function args as defaults.
         """
-        self._kwargs = kwargs
+        self._kwargs = {**kwargs}
 
     def __call__(self, func):
         @functools.wraps(func)
